const 
    gulp_rename = {
        'gulp-util' : 'gutil',
        'gulp-angular-templatecache': 'template'
    },
    gulp        = require('gulp'),
    $           = require('gulp-load-plugins')({rename: gulp_rename}),
    browserSync = require('browser-sync').create(),
    karma       = require('karma').Server,
    pkg         = require('./package.json'),
    del         = require('del');

var banner = [
    '/**',
    ' * @name <%%= pkg.name %>',
    ' * @version v<%%= pkg.version %>',
    ' * @author <%%= pkg.author.name %> <%%= pkg.author.email %>',
    ' * @license <%%= pkg.license %>',
    ' */',
    ''
].join('\n');


/**
 * Concat source files in src folder into one plugin file.
 */
gulp.task('concat:source', ['template:source'], function(){
    return gulp.src(
        [
            'src/common/*.js',
            'src/!(template)**/!(public)*.js',
            'src/public.js',
            'src/template/*.js'
        ])
        .pipe($.insert.append('\n\n'))                  // insert spaces between source files
        .pipe($.concat('<%= mainfile %>'))              // concat into one file
        .pipe($.ngAnnotate({add:true, remove:true}))    // for strict dependecy injection
        .pipe($.wrap('(function(window, angular)'       
                        +'{\n\n<%%=contents%>}'          // wrap myPlugin.js for supporting
                    +')(window, window.angular);'))      // both amd and global
        .pipe(gulp.dest('./dist'));
});



/**
 * Append header to plugin file.
 */
gulp.task('header:source', ['concat:source'], function() {
    return gulp.src('./dist/<%= mainfile %>')
        .pipe($.header(banner, {pkg: pkg}))     // append a header
        .pipe(gulp.dest('./'))
        .pipe($.copy('example/scripts'));       // copy for displaying examples
});



/**
 * Uglify the plugin file and also append header to it.
 */
gulp.task('uglify:source', ['concat:source'], function() {
    return gulp.src('./dist/<%= mainfile %>')
        .pipe($.uglify())
        .pipe($.header(banner, {pkg: pkg}))
        .pipe($.rename({suffix: '.min'}))
        .pipe(gulp.dest('./'));
});


/**
 * Minify htmls in src/template directory and generate template.js which contains the htmls
 */
gulp.task('template:source', function(){
    return gulp.src('./src/template/*.html')
        .pipe($.htmlmin({
            removeComments: true,
            collapseWhitespace: true,
            conservativeCollapse: true,
        }))
        .pipe($.template('template.js', {
            module: '<%= modulename %>'     // your module name
        }))
        .pipe(gulp.dest('./src/template'));
});



/**
 * Delete temporary dist folder
 */
gulp.task('clean:dist', function(){
    return del(['dist']);
});



/**
 * Init a browser-sync with proxy and port.
 */
gulp.task('init:browserSync', function(){
    browserSync.init({
        proxy: 'localhost:8000',
        port: process.env.PORT      //replace this with a port which you want to use
    });
});



/**
 * webserver for examples
 */
gulp.task('webserver:example', ['init:browserSync', 'watch:example'], function(){
    return gulp.src('example')
        .pipe($.webserver({
            host: 'localhost',
            path: '/example',
            port: '8000'
        }))
        .on('error', $.gutil.log);
});



/**
 * webserver for ngdocs
 */
gulp.task('webserver:ngdocs', ['build:ngdocs', 'init:browserSync', 'watch:ngdocs'], function(){
    return gulp.src('docs')
        .pipe($.webserver({
            host: 'localhost',
            path: '/docs',
            port: '8000'
        }))
        .on('error', $.gutil.log);
});



/**
 * Watch source files and reload docs page when detects change
 */
gulp.task('watch:ngdocs', ['init:browserSync'], function(){
    gulp.watch(['src/**/*.js'], ['build', 'build:ngdocs']);
    gulp.watch(['docs/index.html']).on('change', browserSync.reload);
});



/**
 * Generate docs templates with plugin source file
 */
gulp.task('build:ngdocs', ['build'], function(){
    return gulp.src('<%= mainfile %>')
    .pipe($.ngdocs.process({
        html5Mode : false,
        scripts: [
            './bower_components/angular/angular.min.js',
            './bower_components/angular-animate/angular-animate.min.js',
            './bower_components/marked/lib/marked.js',
            './<%=mainfile %>'
        ]
    }))
    .pipe(gulp.dest('./docs'));
});



gulp.task('watch:example', ['init:browserSync'], function() {
    gulp.watch(['src/**/*.js'], ['build']).on('change', browserSync.reload);
    gulp.watch(['example/**/*.html']).on('change', browserSync.reload);
});


gulp.task('test:spec', ['build', 'lint:source'], function (done) {
    new karma({
        configFile: __dirname + '/karma.conf.js',
        browsers: ['PhantomJS'],
        singleRun: true
        }, function(){
            done();
        }
    ).start();
});


gulp.task('lint:source', function (){
    return gulp.src('./src/**/*.js')
    .pipe($.eslint())
    .pipe($.eslint.format())
    .pipe($.eslint.failAfterError());
});


gulp.task('lint:source:nofail', function(){
    return gulp.src('./src/**/*.js')
    .pipe($.eslint())
    .pipe($.eslint.format());
});



gulp.task('watch:source', ['lint:source:nofail'], function(){
    gulp.watch(['src/**/*.js'], ['lint:source:nofail']);
});



gulp.task('build', ['template:source', 'concat:source', 'header:source', 'uglify:source']);

gulp.task('test', ['build', 'lint:source', 'test:spec']);

gulp.task('clean', ['clean:dist']);

gulp.task('default', ['build', 'test', 'lint:source']);

gulp.task('ngdocs', ['build:ngdocs']);